@@ -34,7 +34,6 @@ using namespace std;
 
 
 struct ShouldNotBeCalled : std::exception{
-
   explicit ShouldNotBeCalled(){ }
   virtual ~ShouldNotBeCalled() throw() {}
   virtual const char* what () const noexcept {
@@ -70,6 +69,15 @@ struct InvalidInput : std::exception {
 };
 
 
+struct OutOfVectorSize : std::exception{
+  explicit OutOfVectorSize(){ }
+  virtual ~OutOfVectorSize() throw() {}
+  virtual const char* what () const noexcept {
+      return string("Out of vector size!").c_str();
+  }
+};
+
+
 struct InvalidK : public InvalidInput{
   InvalidK( ):InvalidInput( ){
     this->reason = "k must be at least 2, when using the flag -ibd.";
