@@ -23,13 +23,14 @@
  *
  */
 
+#include <random>
+#include <stdio.h>
+#include <limits>       // std::numeric_limits< double >::min()
+#include <math.h>       // ceil
+#include "global.h"     // dout
+#include "updateHap.hpp"
 #include "mcmc.hpp"
 #include "utility.hpp"
-#include <math.h>       /* ceil */
-#include <random>
-#include "updateHap.hpp"
-#include <stdio.h>
-#include <limits>       /* std::numeric_limits< double >::min() */
 
 McmcSample::McmcSample(){};
 McmcSample::~McmcSample(){};
@@ -46,7 +47,7 @@ McmcMachinery::McmcMachinery(DEploidIO* dEploidIO, McmcSample *mcmcSample, Rando
     this->mcmcEventRg_ = this->hapRg_;
     this->propRg_ = this->hapRg_;
     this->initialHapRg_ = this->hapRg_;
-    this->ibdRg_ = this->hapRg_;
+    //this->ibdRg_ = this->hapRg_;
     //this->mcmcEventRg_ = new MersenneTwister(this->seed_);
     //this->propRg_  = new MersenneTwister(this->seed_);
     //this->initialHapRg_ = new MersenneTwister(this->seed_);
@@ -56,7 +57,7 @@ McmcMachinery::McmcMachinery(DEploidIO* dEploidIO, McmcSample *mcmcSample, Rando
         this->calcMaxIteration( dEploidIO_->nMcmcSample_ , dEploidIO_->mcmcMachineryRate_, dEploidIO_->mcmcBurn_ );
     }
     this->MN_LOG_TITRE = 0.0;
-    this->SD_LOG_TITRE = this->dEploidIO_->parameterSigma();
+    this->SD_LOG_TITRE = (useIBD == true) ? this->dEploidIO_->ibdSigma() : this->dEploidIO_->parameterSigma();
     this->PROP_SCALE = 40.0;
 
     stdNorm_ = new StandNormalRandomSample(this->seed_);
@@ -101,10 +102,9 @@ void McmcMachinery::initializeMcmcChain(bool useIBD){
     }
 
     if ( useIBD == true ){
-        this->initializeIbdEssentials();
+        this->ibdInitializeEssentials();
     }
 
-    this->mcmcSample_->IBDpathChangeAt = vector <double> (this->nLoci());
     this->mcmcSample_->siteOfTwoSwitchOne = vector <double> (this->nLoci());
     this->mcmcSample_->siteOfTwoMissCopyOne = vector <double> (this->nLoci());
     this->mcmcSample_->siteOfTwoSwitchTwo = vector <double> (this->nLoci());
@@ -112,7 +112,6 @@ void McmcMachinery::initializeMcmcChain(bool useIBD){
     this->mcmcSample_->siteOfOneSwitchOne = vector <double> (this->nLoci());
     this->mcmcSample_->siteOfOneMissCopyOne = vector <double> (this->nLoci());
 
-    this->mcmcSample_->currentIBDpathChangeAt = vector <double> (this->nLoci());
     this->mcmcSample_->currentsiteOfTwoSwitchOne = vector <double> (this->nLoci());
     this->mcmcSample_->currentsiteOfTwoMissCopyOne = vector <double> (this->nLoci());
     this->mcmcSample_->currentsiteOfTwoSwitchTwo = vector <double> (this->nLoci());
@@ -243,56 +242,19 @@ vector <double> McmcMachinery::titre2prop(vector <double> & tmpTitre){
 }
 
 
-void McmcMachinery::makeLlkSurf(vector <double> altCount, vector <double> refCount, double scalingConst, double err, size_t gridSize){
-    double pGridSpacing = 1.0 / (double)(gridSize+1);
-    vector <double> pGrid;
-    pGrid.push_back(pGridSpacing);
-    for (size_t i = 1; i < gridSize; i++){
-        pGrid.push_back(pGrid.back() + pGridSpacing);
-    }
-    assert(pGrid.size() == gridSize);
-
-    assert(llkSurf.size() == 0);
-
-    for ( size_t i = 0 ; i < altCount.size(); i++) {
-        double alt = altCount[i];
-        double ref = refCount[i];
-
-        vector <double> ll;
-        for ( double unadjustedP : pGrid ){
-            ll.push_back(calcLLK( ref, alt, unadjustedP, err, scalingConst));
-        }
-
-        double llmax = max_value(ll);
-        vector <double> ln;
-        for ( double lltmp : ll ){
-            ln.push_back(exp(lltmp-llmax));
-        }
-
-        double lnSum = sumOfVec(ln);
-        for (size_t i = 0; i < ln.size(); i++){
-            ln[i] = ln[i]/lnSum;
-        }
-
-        vector <double> tmpVec1 = vecProd(ln, pGrid);
-        double mn = sumOfVec(tmpVec1);
-        vector <double> pGridSq = vecProd(pGrid, pGrid);
-        vector <double> tmpVec2 = vecProd(ln, pGridSq);
-        double vr = sumOfVec(tmpVec2) - mn*mn;
-
-        double comm = (mn*(1.0-mn)/vr-1.0);
-        llkSurf.push_back(vector <double> {mn*comm, (1-mn)*comm});
-    }
-    assert(llkSurf.size() == this->nLoci());
-}
 
 
 void McmcMachinery::initializePropIBD(){
     //#Initialise titres and convert to proportions
-    this->currentTitre_ = vector <double> (this->kStrain(), 0.0);
+    //this->initializeTitre();
     this->currentProp_ = ( this->dEploidIO_ -> initialPropWasGiven()) ?
-                          this->dEploidIO_ ->initialProp :
-                          vector <double> (this->kStrain(), 1.0/(double)kStrain());
+                          this->dEploidIO_ ->initialProp:
+                          this->titre2prop( this->currentTitre_ );
+
+    //this->currentTitre_ = vector <double> (this->kStrain(), 0.0);
+    //this->currentProp_ = ( this->dEploidIO_ -> initialPropWasGiven()) ?
+                          //this->dEploidIO_ ->initialProp :
+                          //vector <double> (this->kStrain(), 1.0/(double)kStrain());
 }
 
 
@@ -315,10 +277,9 @@ void McmcMachinery::runMcmcChain( bool showProgress, bool useIBD, bool notInR ){
 
     this->writeLastFwdProb(useIBD);
 
-    this->dEploidIO_->filnalProp = this->mcmcSample_->proportion.back();
+    this->dEploidIO_->finalProp = this->mcmcSample_->proportion.back();
 
     for (size_t atSiteI = 0; atSiteI < nLoci(); atSiteI++ ){
-        this->mcmcSample_->IBDpathChangeAt[atSiteI] /= (double)this->maxIteration_;
         this->mcmcSample_->siteOfTwoSwitchOne[atSiteI] /= (double)this->maxIteration_;
         this->mcmcSample_->siteOfTwoMissCopyOne[atSiteI] /= (double)this->maxIteration_;
         this->mcmcSample_->siteOfTwoSwitchTwo[atSiteI] /= (double)this->maxIteration_;
@@ -332,6 +293,13 @@ void McmcMachinery::runMcmcChain( bool showProgress, bool useIBD, bool notInR ){
     }
 
     if ( useIBD == true ){
+        for (size_t atSiteI = 0; atSiteI < nLoci(); atSiteI++ ){
+            this->ibdPath.IBDpathChangeAt[atSiteI] /= (double)this->maxIteration_;
+        }
+        //vector < vector <double> > reshapedProbs = this->reshapeFm(hprior.stateIdx);
+        //this->dEploidIO_->ibdProbsHeader = getIBDprobsHeader();
+        //this->dEploidIO_->ibdProbsIntegrated = getIBDprobsIntegrated(reshapedProbs);
+        //this->dEploidIO_->writeIBDpostProb(reshapedProbs, this->dEploidIO_->ibdProbsHeader);
         clog << "Proportion update acceptance rate: "<<acceptUpdate / (this->kStrain()*1.0*this->maxIteration_)<<endl;
         this->dEploidIO_->initialProp = averageProportion(this->mcmcSample_->proportion);
         this->dEploidIO_->setInitialPropWasGiven(true);
@@ -348,6 +316,8 @@ void McmcMachinery::runMcmcChain( bool showProgress, bool useIBD, bool notInR ){
 }
 
 
+
+
 void McmcMachinery::computeDiagnostics(){
     //clog << "Proportion update acceptance rate: "<<acceptUpdate / (this->kStrain()*1.0*this->maxIteration_)<<endl;
     this->dEploidIO_->setacceptRatio(acceptUpdate / (1.0*this->maxIteration_));
@@ -406,7 +376,7 @@ vector <double> McmcMachinery::averageProportion(vector < vector <double> > &pro
 void McmcMachinery::sampleMcmcEvent( bool useIBD ){
     this->recordingMcmcBool_ = ( currentMcmcIteration_ > this->mcmcThresh_ && currentMcmcIteration_ % this->McmcMachineryRate_ == 0 );
     if ( useIBD == true ){
-        sampleMcmcEventIbdStep();
+        ibdSampleMcmcEventStep();
         assert(doutProp());
     } else {
         this->eventInt_ = this->mcmcEventRg_->sampleInt(3);
@@ -427,209 +397,67 @@ void McmcMachinery::sampleMcmcEvent( bool useIBD ){
 }
 
 
-vector <size_t> McmcMachinery::findWhichIsSomething(vector <size_t> tmpOp, size_t something){
-    vector <size_t> ret;
-    for ( size_t i = 0; i < tmpOp.size(); i++){
-        if ( tmpOp[i] == something){
-            ret.push_back(i);
-        }
-    }
-    return ret;
-}
 
 
-void McmcMachinery::makeIbdTransProbs(){
-    size_t nPattern = hprior.nPattern();
-    size_t nState = hprior.nState();
-    assert(ibdTransProbs.size() == 0);
-
-    for ( size_t i = 0; i < nPattern; i++ ){
-        vector <double> transProbRow(nState);
-        vector <size_t> wi = findWhichIsSomething(hprior.stateIdx, i);
-        for (size_t wii : wi){
-            transProbRow[wii] = 1;
-        }
-        ibdTransProbs.push_back(transProbRow);
-    }
-}
 
 
-void McmcMachinery::computeUniqueEffectiveKCount(){
-    this->uniqueEffectiveKCount = vector <int> (this->kStrain());
-    for (size_t effectiveKtmp : this->hprior.effectiveK) {
-        int effectiveKidx = effectiveKtmp-1;
-        assert(effectiveKidx>=0);
-        this->uniqueEffectiveKCount[effectiveKidx]++;
-    }
-}
 
 
-vector <double> McmcMachinery::computeStatePrior( double theta ){
-    //#Calculate state prior given theta (theta is prob IBD)
-    vector <double> pr0(this->kStrain());
-    for (int i = 0; i < (int)pr0.size(); i++){
-        pr0[i] = binomialPdf(i, (int)(this->kStrain()-1), theta);
-    }
 
-    vector <double> effectiveKPrior;
-    for ( size_t effectiveKtmp : this->hprior.effectiveK){
-        int effectiveKidx = effectiveKtmp-1;
-        assert(effectiveKidx >= 0);
-        assert(effectiveKidx < (int)this->kStrain());
-        effectiveKPrior.push_back(pr0[effectiveKidx]/uniqueEffectiveKCount[effectiveKidx]);
-    }
+void McmcMachinery::ibdInitializeEssentials(){
 
-    vector <double> ret;
-    for (size_t stateIdxTmp : this->hprior.stateIdx){
-        ret.push_back(effectiveKPrior[stateIdxTmp]);
-    }
-    return ret;
-}
-
-
-void McmcMachinery::initializeIbdEssentials(){
-    // initialize haplotype prior
-    this->hprior.buildHprior(this->kStrain(), this->dEploidIO_->plaf_);
-    this->hprior.transposePriorProbs();
-
-    // compute likelihood surface
-    this->makeLlkSurf(this->dEploidIO_->altCount_, this->dEploidIO_->refCount_);
+    this->initializePropIBD();
+    this->ibdPath.init(*this->dEploidIO_, this->hapRg_);
 
     vector <double> llkOfData;
     for ( size_t i = 0; i < nLoci(); i++){
         double wsaf = this->dEploidIO_->altCount_[i] / (this->dEploidIO_->refCount_[i] + this->dEploidIO_->altCount_[i] + 0.00000000000001);
         double adjustedWsaf = wsaf*(1-0.01) + (1-wsaf)*0.01;
-        llkOfData.push_back( logBetaPdf(adjustedWsaf, this->llkSurf[i][0], this->llkSurf[i][1]));
+        llkOfData.push_back( logBetaPdf(adjustedWsaf, this->ibdPath.llkSurf[i][0], this->ibdPath.llkSurf[i][1]));
     }
     dout << "LLK of data = " << sumOfVec(llkOfData) << endl;
 
-    this->initializePropIBD();
-    this->setTheta(1.0 / (double)kStrain());
-    this->makeIbdTransProbs();
-    this->computeUniqueEffectiveKCount();
-
-    // initialize fm
-    this->fSumState = vector <double> (this->hprior.nPattern());
-
-    // initialize ibdPath
-    this->ibdPath = vector <size_t> (this->nLoci());
 }
 
 
-vector <double> McmcMachinery::computeLlkOfStatesAtSiteI( size_t siteI, double err ){
-    vector <double> llks;
-    for ( vector <int> hSetI : this->hprior.hSet ){
-        double qs = 0;
-        for ( size_t j = 0; j < this->kStrain() ; j++ ){
-            qs += (double)hSetI[j] * this->currentProp_[j];
-        }
-        double qs2 = qs*(1-err) + (1-qs)*err ;
-        llks.push_back(logBetaPdf(qs2, this->llkSurf[siteI][0], this->llkSurf[siteI][1]));
-    }
 
-    double maxllk = max_value(llks);
-    vector <double> ret;
-    for ( double llk : llks ){
-        double normalized = exp(llk-maxllk);
-        if ( normalized == 0 ){
-            normalized = std::numeric_limits< double >::min();
-        }
-        ret.push_back(normalized);
-    }
-
-    return ret;
-}
-
-
-void McmcMachinery::updateFmAtSiteI(vector <double> & prior, vector <double> & llk){
-    vector <double> postAtSiteI = vecProd(prior, llk);
-    normalizeByMax(postAtSiteI);
-    this->fm.push_back(postAtSiteI);
-    this->fSum = sumOfVec(postAtSiteI);
-    for ( size_t i = 0; i < fSumState.size(); i++){
-        this->fSumState[i] = 0;
-        for ( size_t j = 0; j < hprior.nState(); j++ ){
-            this->fSumState[i] += ibdTransProbs[i][j]*postAtSiteI[j];
-        }
-    }
-}
-
-
-vector <double> McmcMachinery::computeLlkAtAllSites(double err){
-    vector <double > ret;
-    for ( size_t site = 0; site < this->nLoci(); site++ ){
-        double qs = 0;
-        for ( size_t j = 0; j < this->kStrain() ; j++ ){
-            qs += (double)this->currentHap_[site][j] * this->currentProp_[j];
-        }
-        double qs2 = qs*(1-err) + (1-qs)*err ;
-        ret.push_back(logBetaPdf(qs2, this->llkSurf[site][0], this->llkSurf[site][1]));
-    }
-    return ret;
-}
-
-
-void McmcMachinery::sampleMcmcEventIbdStep(){
-    this->fm.clear();
-    double pRecomb = 0.01; // This should be parsed in
-    double pNoRecomb = 0.99;
-    vector <double> statePrior = this->computeStatePrior(this->theta());
-    vector <double> vPrior = vecProd(statePrior, this->hprior.priorProbTrans[0]);
-
-    vector <double> lk = computeLlkOfStatesAtSiteI(0);
-    this->updateFmAtSiteI(vPrior, lk);
-    for ( size_t siteI = 1; siteI < this->nLoci(); siteI++ ){
-        vector <double> vNoRec;
-        for ( size_t stateIdxTmp : hprior.stateIdx ){
-            vNoRec.push_back(this->fSumState[stateIdxTmp]);
-        }
-
-        for ( size_t i = 0; i < hprior.nState(); i++ ){
-            vPrior[i] = (vNoRec[i] * pNoRecomb + fSum * pRecomb * statePrior[i]) * hprior.priorProbTrans[siteI][i];
-        }
-
-        lk = computeLlkOfStatesAtSiteI(siteI);
-        this->updateFmAtSiteI(vPrior, lk);
-    }
+void McmcMachinery::ibdSampleMcmcEventStep(){
+    vector <double> effectiveKPrior = this->ibdPath.computeEffectiveKPrior(this->ibdPath.theta());
+    vector <double> statePrior = this->ibdPath.computeStatePrior(effectiveKPrior);
+    // First building the path likelihood
+    this->ibdPath.computeIbdPathFwdProb(this->currentProp_, statePrior);
 
     ////#Now sample path given matrix
-    int lociIdx = this->nLoci()-1;
-    vector <double> tmpProp = fm[lociIdx];
-    (void)normalizeBySum(tmpProp);
-    ibdPath[lociIdx] = sampleIndexGivenProp(this->ibdRg_, tmpProp);
-
-    assert(this->fm.size() == nLoci());
-    while ( lociIdx > 0 ){
-        lociIdx--;
-        vector <double> vNoRecomb = vecProd(this->ibdTransProbs[this->hprior.stateIdx[ibdPath[lociIdx+1]]], fm[lociIdx]);
-        assert(vNoRecomb.size() == this->hprior.nState());
-        vector <double> vRecomb = fm[lociIdx];
-        assert(vRecomb.size() == this->hprior.nState());
-        vector <double> prop (this->hprior.nState());
-        for ( size_t i = 0; i < prop.size(); i++){
-            prop[i] = vNoRecomb[i]*pNoRecomb + vRecomb[i]*pRecomb*statePrior[ibdPath[lociIdx+1]];
-        }
-        tmpProp = prop;
-        (void)normalizeBySum(tmpProp);
-        ibdPath[lociIdx] = sampleIndexGivenProp(this->ibdRg_, tmpProp);
-        assert( ibdPath[lociIdx] < this->hprior.nState() );
-        assert( ibdPath[lociIdx] >= 0 );
-    }
+    this->ibdPath.ibdSamplePath(statePrior);
 
     //#Get haplotypes and update LLK for each site
+    this->ibdUpdateHaplotypesFromPrior();
+    vector <double> llkAtAllSites = computeLlkAtAllSites();
+    ////#Given current haplotypes, sample titres 1 by 1 using MH
+    this->ibdUpdateProportionGivenHap(llkAtAllSites);
+    // Compute new theta after all proportion and haplotypes are up to date.
+    this->ibdPath.computeAndUpdateTheta();
+
+    this->currentLLks_ = llkAtAllSites;
+    this->currentExpectedWsaf_ = this->calcExpectedWsaf( this->currentProp_ );
+}
+
+
+void McmcMachinery::ibdUpdateHaplotypesFromPrior(){
     for (size_t i = 0; i < this->nLoci(); i++){
         for ( size_t j = 0; j < kStrain(); j++){
-            this->currentHap_[i][j] = (double)this->hprior.hSet[ibdPath[i]][j];
+            this->currentHap_[i][j] = (double)this->ibdPath.hprior.hSet[ibdPath.ibdConfigurePath[i]][j];
         }
     }
+}
 
-    vector <double> llkAtAllSites = computeLlkAtAllSites();
 
-    ////#Given current haplotypes, sample titres 1 by 1 using MH
+void McmcMachinery::ibdUpdateProportionGivenHap(vector <double> &llkAtAllSites){
     for (size_t i = 0; i < kStrain(); i++){
         double v0 = this->currentTitre_[i];
         vector <double> oldProp = this->currentProp_;
-        this->currentTitre_[i] += (this->stdNorm_->genReal() * 0.1 + 0.0); // tit.0[i]+rnorm(1, 0, scale.t.prop);
+        //this->currentTitre_[i] += (this->stdNorm_->genReal() * 0.1 + 0.0); // tit.0[i]+rnorm(1, 0, scale.t.prop);
+        this->currentTitre_[i] += (this->stdNorm_->genReal() * SD_LOG_TITRE* 1.0/PROP_SCALE + 0.0); // tit.0[i]+rnorm(1, 0, scale.t.prop);
         this->currentProp_ = this->titre2prop(this->currentTitre_);
         vector <double> vv = computeLlkAtAllSites();
         double rr = normal_pdf( this->currentTitre_[i], 0, 1) /
@@ -643,42 +471,25 @@ void McmcMachinery::sampleMcmcEventIbdStep(){
             this->currentProp_ = oldProp;
         }
     }
-
-    this->computeAndUpdateTheta();
-
-    this->currentLLks_ = llkAtAllSites;
-    this->currentExpectedWsaf_ = this->calcExpectedWsaf( this->currentProp_ );
 }
 
 
-void McmcMachinery::computeAndUpdateTheta(){
-    vector <size_t> obsState;
-    size_t previousState = 0;
-    size_t atSiteI = 0;
-    for (size_t a : ibdPath){
-        if ( a != previousState ){
-            obsState.push_back(a);
+vector <double> McmcMachinery::computeLlkAtAllSites(double err){
+    vector <double > ret;
+    for ( size_t site = 0; site < this->nLoci(); site++ ){
+        double qs = 0;
+        for ( size_t j = 0; j < this->kStrain() ; j++ ){
+            qs += (double)this->currentHap_[site][j] * this->currentProp_[j];
         }
-        if ( this->hprior.stateIdx[a] != this->hprior.stateIdx[previousState] ){
-            this->mcmcSample_->IBDpathChangeAt[atSiteI] += 1.0;
-            this->mcmcSample_->currentIBDpathChangeAt[atSiteI] = 1.0;
-        } else {
-            this->mcmcSample_->currentIBDpathChangeAt[atSiteI] = 0.0;
-        }
-        previousState = a;
-        atSiteI++;
+        double qs2 = qs*(1-err) + (1-qs)*err ;
+        ret.push_back(logBetaPdf(qs2, this->ibdPath.llkSurf[site][0], this->ibdPath.llkSurf[site][1]));
     }
-
-    size_t sumOfKeffStates = 0;
-    size_t sccs = 0;
-    for (size_t obs : obsState){
-        sumOfKeffStates += this->hprior.effectiveK[obs] - 1;
-        sccs += this->kStrain() - this->hprior.effectiveK[obs];
-    }
-    this->setTheta(rBeta(sccs+1.0, sumOfKeffStates+1.0, this->propRg_));
+    return ret;
 }
 
 
+
+
 vector <double> McmcMachinery::calcExpectedWsaf( vector <double> &proportion ){
     //assert ( sumOfVec(proportion) == 1.0); // this fails ...
     vector <double> expectedWsaf (this->nLoci_, 0.0);
